@@ -73,8 +73,40 @@ class NotebookMovieV2:
         # PERFORMANCE FIX: Use cached fixed limits instead of recalculating
         cached_data = self._cached_limits[session_key]
         
-        if 'x' in df.columns and 'y' in df.columns:
-            # 2D projection
+        if all(col in df.columns for col in ['x', 'y', 'z']):
+            # TRUE 3D VISUALIZATION - the star of the show!
+            from mpl_toolkits.mplot3d import Axes3D
+            import numpy as np
+            
+            # Clear the axis and recreate as 3D if needed
+            if not hasattr(ax, 'zaxis'):
+                ax.remove()
+                ax = ax.figure.add_subplot(ax.get_geometry()[0], ax.get_geometry()[1], 
+                                           ax.get_geometry()[2], projection='3d')
+            
+            scatter = ax.scatter(df['x'], df['y'], df['z'], alpha=0.7, s=25, c='steelblue')
+            ax.set_xlabel('Dimension 1', fontweight='bold')
+            ax.set_ylabel('Dimension 2', fontweight='bold')
+            ax.set_zlabel('Dimension 3', fontweight='bold')
+            
+            # Set equal aspect ratio for better 3D visualization
+            if len(df) > 0:
+                max_range = np.max([
+                    np.max(df['x']) - np.min(df['x']),
+                    np.max(df['y']) - np.min(df['y']),
+                    np.max(df['z']) - np.min(df['z'])
+                ])
+                
+                mid_x = (np.max(df['x']) + np.min(df['x'])) * 0.5
+                mid_y = (np.max(df['y']) + np.min(df['y'])) * 0.5
+                mid_z = (np.max(df['z']) + np.min(df['z'])) * 0.5
+                
+                ax.set_xlim(mid_x - max_range/2, mid_x + max_range/2)
+                ax.set_ylim(mid_y - max_range/2, mid_y + max_range/2)
+                ax.set_zlim(mid_z - max_range/2, mid_z + max_range/2)
+                
+        elif 'x' in df.columns and 'y' in df.columns:
+            # 2D projection fallback
             ax.scatter(df['x'], df['y'], alpha=0.6, s=20)
             ax.set_xlabel('Dimension 1', fontweight='bold')
             ax.set_ylabel('Dimension 2', fontweight='bold')
@@ -83,23 +115,6 @@ class NotebookMovieV2:
             if cached_data['xlim'] and cached_data['ylim']:
                 ax.set_xlim(cached_data['xlim'])
                 ax.set_ylim(cached_data['ylim'])
-                
-        elif all(col in df.columns for col in ['x', 'y', 'z']):
-            # 3D projection - project to 2D for display
-            scatter = ax.scatter(df['x'], df['y'], alpha=0.6, s=20, c=df['z'], cmap='viridis')
-            ax.set_xlabel('Dimension 1', fontweight='bold')
-            ax.set_ylabel('Dimension 2', fontweight='bold')
-            
-            # SCALING FIX: Use fixed axis limits
-            if cached_data['xlim'] and cached_data['ylim']:
-                ax.set_xlim(cached_data['xlim'])
-                ax.set_ylim(cached_data['ylim'])
-            
-            # Add colorbar for Z dimension
-            try:
-                plt.colorbar(scatter, ax=ax, label='Dimension 3')
-            except:
-                pass  # Ignore colorbar errors
         
         ax.set_title(f'Embedding Space - Epoch {current_epoch}', fontweight='bold')
         ax.grid(True, alpha=0.3)
@@ -229,8 +244,46 @@ class NotebookMovieV2:
                     if len(df) > sample_size:
                         df = df.sample(sample_size, random_state=42)
                     
-                    if 'x' in df.columns and 'y' in df.columns:
-                        # Enhanced scatter plot
+                    if all(col in df.columns for col in ['x', 'y', 'z']):
+                        # TRUE 3D VISUALIZATION - the star of the show!
+                        from mpl_toolkits.mplot3d import Axes3D
+                        import numpy as np
+                        
+                        # Clear the axis and recreate as 3D if needed
+                        if not hasattr(ax, 'zaxis'):
+                            ax.remove()
+                            ax = ax.figure.add_subplot(ax.get_geometry()[0], ax.get_geometry()[1], 
+                                                       ax.get_geometry()[2], projection='3d')
+                        
+                        if color_by and color_by in df.columns:
+                            scatter = ax.scatter(df['x'], df['y'], df['z'], alpha=0.7, s=25, 
+                                               c=df[color_by], cmap='viridis')
+                        else:
+                            scatter = ax.scatter(df['x'], df['y'], df['z'], alpha=0.7, s=25, 
+                                               c='steelblue')
+                        
+                        ax.set_xlabel('Dimension 1', fontsize=10)
+                        ax.set_ylabel('Dimension 2', fontsize=10)
+                        ax.set_zlabel('Dimension 3', fontsize=10)
+                        
+                        # Set equal aspect ratio for better 3D visualization
+                        if len(df) > 0:
+                            max_range = np.max([
+                                np.max(df['x']) - np.min(df['x']),
+                                np.max(df['y']) - np.min(df['y']),
+                                np.max(df['z']) - np.min(df['z'])
+                            ])
+                            
+                            mid_x = (np.max(df['x']) + np.min(df['x'])) * 0.5
+                            mid_y = (np.max(df['y']) + np.min(df['y'])) * 0.5
+                            mid_z = (np.max(df['z']) + np.min(df['z'])) * 0.5
+                            
+                            ax.set_xlim(mid_x - max_range/2, mid_x + max_range/2)
+                            ax.set_ylim(mid_y - max_range/2, mid_y + max_range/2)
+                            ax.set_zlim(mid_z - max_range/2, mid_z + max_range/2)
+                        
+                    elif 'x' in df.columns and 'y' in df.columns:
+                        # 2D projection fallback
                         if color_by and color_by in df.columns:
                             scatter = ax.scatter(df['x'], df['y'], alpha=0.7, s=25, 
                                                c=df[color_by], cmap='viridis')
@@ -316,23 +369,30 @@ class NotebookMovieV2:
         def update_movie(epoch=1):
             """Update movie display for given epoch."""
             try:
-                # Create subplot layout
+                # Create subplot layout - EMBEDDING SPACE IS THE STAR!
                 if show_embedding_evolution and show_loss_evolution:
-                    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(16, 6))
+                    fig = plt.figure(figsize=(16, 8))
+                    ax2 = fig.add_subplot(1, 2, 1, projection='3d')  # Large 3D embedding plot
+                    ax1 = fig.add_subplot(1, 2, 2)  # Smaller loss plot
                 elif show_loss_evolution:
                     fig, ax1 = plt.subplots(1, 1, figsize=(10, 6))
                     ax2 = None
                 else:
-                    fig, ax2 = plt.subplots(1, 1, figsize=(8, 6))
+                    fig = plt.figure(figsize=(12, 8))
+                    ax2 = fig.add_subplot(1, 1, 1, projection='3d')  # Full 3D embedding plot
                     ax1 = None
                 
-                # Plot loss evolution up to current epoch
-                if show_loss_evolution and ax1 is not None:
-                    self.client._plot_loss_evolution_frame(ax1, loss_history, training_info, epoch)
-                
-                # Plot embedding evolution for current epoch (USING IMPROVED V2 METHOD)
+                # Plot embedding evolution for current epoch (USING IMPROVED V2 METHOD) - THE STAR!
                 if show_embedding_evolution and ax2 is not None:
                     self.plot_embedding_evolution_frame_v2(ax2, epoch_projections, epoch)
+                    ax2.set_title('🌌 Featrix Sphere - 3D Embedding Space', fontweight='bold', fontsize=14)
+                
+                # Plot loss evolution up to current epoch - as sparkline
+                if show_loss_evolution and ax1 is not None:
+                    self.client._plot_loss_evolution_frame(ax1, loss_history, training_info, epoch)
+                    ax1.set_title('📊 Training Loss', fontweight='bold', fontsize=12)
+                    ax1.tick_params(axis='both', which='major', labelsize=10)
+                    ax1.grid(True, alpha=0.3)
                 
                 plt.tight_layout()
                 plt.show()
